import os

import boto3

ENVNAME_AWS_ACCESS_KEY = "SECRETKEEPER_AWS_ACCESS_KEY"
ENVNAME_AWS_SECRET_KEY = "SECRETKEEPER_AWS_SECRET_KEY"
ENVNAME_AWS_REGION = "SECRETKEEPER_AWS_REGION"


def _get_client():
    access_key = os.environ[ENVNAME_AWS_ACCESS_KEY]
    secret_key = os.environ[ENVNAME_AWS_SECRET_KEY]
    region = os.environ[ENVNAME_AWS_REGION]

    client = boto3.client(
        'ssm',
        region_name=region,
        aws_access_key_id=access_key,
        aws_secret_access_key=secret_key,
    )
    return client


def tell(alias):
    response = _get_client().get_parameter(
        Name=alias, WithDecryption=True,
    )
    return response['Parameter']['Value']


def tell_safe(alias):
    try:
        return tell(alias)
    except Exception:
        return None


__all__ = ["tell", "tell_safe", "ENVNAME_AWS_ACCESS_KEY", "ENVNAME_AWS_SECRET_KEY", "ENVNAME_AWS_REGION"]



import unittest
try:
    from unittest import mock
except ImportError:
    import mock

from botocore.exceptions import ClientError

from ridi import secret_keeper


class FakeClient(object):
    def __init__(self, dict):
        self._dict = dict

    def get_parameter(self, Name, WithDecryption):
        assert WithDecryption is True
        if Name in self._dict:
            return {
                'Parameter': {
                    'Name': Name,
                    'Type': "SecureString",
                    'Value': self._dict[Name],
                }
            }
        else:
            error_response = {
                'Error': {
                    'Code': "ParameterNotFound",
                },
            }
            raise ClientError(error_response, operation_name="GetParameter")


class TestSecretKeeperBase(unittest.TestCase):
    def setUp(self):
        _client = FakeClient({
            "ones": "11111",
        })
        self._patcher = mock.patch('ridi.secret_keeper._get_client', return_value=_client)
        self._patcher.start()

    def tearDown(self):
        self._patcher.stop()


class TestSecretKeeperGet(TestSecretKeeperBase):
    def test_existing_keys(self):
        self.assertEqual(secret_keeper.tell("ones"), "11111")

    def test_non_existing_keys(self):
        with self.assertRaises(ClientError):
            secret_keeper.tell("twos")


class TestSecretKeeperGetSafe(TestSecretKeeperBase):
    def test_existing_keys(self):
        self.assertEqual(secret_keeper.tell_safe("ones"), "11111")

    def test_non_existing_keys(self):
        self.assertEqual(secret_keeper.tell_safe("twos"), None)




"""A setuptools based setup module.
"""

from setuptools import setup, find_packages

setup(
    name='secret-keeper',
    packages=[
        'ridi.secret_keeper',
    ],
    version='0.1.0',
    description='Secret Keeper',
    url='https://github.com/ridi/secret-keeper-python',
    keywords=['secret', 'secret-keeper', 'ridi', 'ridibooks'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    install_requires=[
        'boto3>=1.9.16',
    ],
)